import Vue from 'vue';
import VueRouter from 'vue-router';
// @ts-ignore
import App from '@@/plugin-vue/App.vue';
import { IRoute } from '../index';

interface IOpts {
  rootElement: string;
  routes: IRoute[];
  history: any;
  mode: string;
  rootRoutes?: IRoute[];
  extraProps?: object;
  plugin: Plugin
}

interface RenderRoutesProps {
  routes?: IRoute[];
  extraProps?: object;
  rootRoutes?: IRoute[];
}

function renderRoutes(opts: RenderRoutesProps): IRoute[] {
  if (opts.routes) {
    return opts.routes.map((route: IRoute, index: number) => {
      const passProps = {
        key: route.key || index,
        path: route.path?.toLowerCase(),
        name: route.path?.toLowerCase(),
        component: route.component,
      };

      if (route.redirect) {
        return {
          ...passProps,
          redirect: route.redirect,
        };
      } else {
        return {
          ...passProps,
          children: renderRoutes({ ...opts, routes: route.routes }),
        };
      }
    });
  }

  return [];
}

function createRouter(opts: IOpts, routes: any) {
  Vue.use(VueRouter);

  let mode = 'history';

  if (opts.mode === 'browser') {
    mode = 'history';
  } else if (opts.mode === 'hash') {
    mode = 'hash';
  } else if (opts.mode === 'memory') {
    mode = 'abstract';
  }

  const router = new VueRouter({
    // @ts-ignore
    mode,
    routes,
  });

  return router;
}

export default function renderClient(opts: IOpts) {
  const container = App;

  if (opts.rootElement) {
    const routes = renderRoutes({
      routes: opts.routes,
    });

    new Vue({
      router: createRouter(opts, routes),
      render: (h: any) => h(container),
    }).$mount(`#${opts.rootElement}`);
  } else {
    return container;
  }
}
